@@ -12,7 +12,7 @@ use strict;
 @ISA = qw(Verilog::Netlist::ContAssign::Struct
 	Verilog::Netlist::Subclass);
 
-$VERSION = '3.223';
+$VERSION = '3.231';
 
 structs('new',
 	'Verilog::Netlist::ContAssign::Struct'
@@ -59,7 +59,7 @@ sub verilog_text {
 sub dump {
     my $self = shift;
     my $indent = shift||0;
-    print " "x$indent,"ContAssign:",$self->keyword,"  lhs:",$self->lhs,"  rhs",$self->rhs;
+    print " "x$indent,"ContAssign:",$self->keyword,"  lhs:",$self->lhs,"  rhs:",$self->rhs;
     print "\n";
 }
 
@@ -134,7 +134,7 @@ Verilog-Perl is part of the L<http://www.veripool.org/> free Verilog EDA
 software tool suite.  The latest version is available from CPAN and from
 L<http://www.veripool.org/verilog-perl>.
 
-Copyright 2000-2009 by Wilson Snyder.  This package is free software; you
+Copyright 2000-2010 by Wilson Snyder.  This package is free software; you
 can redistribute it and/or modify it under the terms of either the GNU
 Lesser General Public License Version 3 or the Perl Artistic License Version 2.0.
 
