@@ -3,6 +3,7 @@
 ######################################################################
 
 package Verilog::Netlist::ModPort;
+use Class::Struct;
 
 use Verilog::Netlist;
 use Verilog::Netlist::Net;
@@ -12,7 +13,7 @@ use strict;
 @ISA = qw(Verilog::Netlist::ModPort::Struct
 	Verilog::Netlist::Subclass);
 
-$VERSION = '2.468';
+$VERSION = '3.231';
 
 structs('new',
 	'Verilog::Netlist::ModPort::Struct'
@@ -109,10 +110,10 @@ sub new_net {
     my $self = shift;
     # @_ params
     # Create a new net under this
-    my $netref = new Verilog::Netlist::Net(direction=>'net', data_type=>'wire',
-					   @_,
-					   module=>$self, );
-    $self->_nets($netref->name(), $netref);
+    my $netref = new Verilog::Netlist::Net (direction=>'net', data_type=>'wire',
+					    @_,
+					    module=>$self, );
+    $self->_nets ($netref->name(), $netref);
     return $netref;
 }
 
@@ -120,8 +121,8 @@ sub new_port {
     my $self = shift;
     # @_ params
     # Create a new port under this module
-    my $portref = new Verilog::Netlist::Port(@_, module=>$self,);
-    $self->_ports($portref->name(), $portref);
+    my $portref = new Verilog::Netlist::Port (@_, module=>$self,);
+    $self->_ports ($portref->name(), $portref);
     return $portref;
 }
 
@@ -264,8 +265,7 @@ sorted by name.
 =item $self->verilog_text
 
 Returns verilog code which represents this modport.  Returned as an array
-that must be joined together to form the final text string.  The netlist
-must be already ->link'ed for this to work correctly.
+that must be joined together to form the final text string.
 
 =back
 
@@ -275,7 +275,7 @@ Verilog-Perl is part of the L<http://www.veripool.org/> free Verilog EDA
 software tool suite.  The latest version is available from CPAN and from
 L<http://www.veripool.org/verilog-perl>.
 
-Copyright 2000-2019 by Wilson Snyder.  This package is free software; you
+Copyright 2000-2010 by Wilson Snyder.  This package is free software; you
 can redistribute it and/or modify it under the terms of either the GNU
 Lesser General Public License Version 3 or the Perl Artistic License
 Version 2.0.
