@@ -14,7 +14,7 @@ use base qw(DynaLoader);
 use strict;
 use vars qw($VERSION $Debug);
 
-$VERSION = '3.223';
+$VERSION = '3.231';
 
 #$Debug sets the default value for debug.  You're better off with the object method though.
 
@@ -88,11 +88,17 @@ sub new {
     return $self;
 }
 
+sub DESTROY {
+    my $self = shift;
+    $self->_DESTROY;
+}
+
 ######################################################################
 ####  Accessors
 
 sub callback_names {
-    return sort @_Callback_Names;
+    my @out = sort @_Callback_Names;
+    return @out;
 }
 
 sub debug {
@@ -511,7 +517,7 @@ Verilog-Perl is part of the L<http://www.veripool.org/> free Verilog EDA
 software tool suite.  The latest version is available from CPAN and from
 L<http://www.veripool.org/verilog-perl>.
 
-Copyright 2000-2009 by Wilson Snyder.  This package is free software; you
+Copyright 2000-2010 by Wilson Snyder.  This package is free software; you
 can redistribute it and/or modify it under the terms of either the GNU
 Lesser General Public License Version 3 or the Perl Artistic License Version 2.0.
 
