@@ -1,7 +1,7 @@
 // -*- C++ -*-
 //*************************************************************************
 //
-// Copyright 2000-2009 by Wilson Snyder.  This program is free software;
+// Copyright 2000-2010 by Wilson Snyder.  This program is free software;
 // you can redistribute it and/or modify it under the terms of either the GNU
 // Lesser General Public License Version 3 or the Perl Artistic License Version 2.0.
 //
@@ -27,6 +27,17 @@
 #include <vector>
 #include <map>
 #include <cassert>
+#include <cerrno>
+
+#include <sys/types.h>
+#include <sys/stat.h>
+#include <fcntl.h>
+
+#if defined(_WIN32) && !defined(__MINGW32__) && !defined(__CYGWIN__)
+# include <io.h>
+#else
+# include <unistd.h>
+#endif
 
 #include "VPreproc.h"
 #include "VPreprocLex.h"
@@ -42,7 +53,7 @@ class VPreDefRef {
     string	m_name;		// Define last name being defined
     string	m_params;	// Define parameter list for next expansion
     string	m_nextarg;	// String being built for next argument
-    int		m_parenLevel;	// Parenthesis counting inside def args
+    int		m_parenLevel;	// Parenthesis counting inside def args (for PARENT not child)
 
     vector<string> m_args;	// List of define arguments
 public:
@@ -51,9 +62,10 @@ public:
     string nextarg() const { return m_nextarg; }
     void nextarg(const string& value) { m_nextarg = value; }
     int parenLevel() const { return m_parenLevel; }
+    void parenLevel(int value) { m_parenLevel = value; }
     vector<string>& args() { return m_args; }
-    VPreDefRef(const string& name, const string& params, int pl)
-	: m_name(name), m_params(params), m_parenLevel(pl) {}
+    VPreDefRef(const string& name, const string& params)
+	: m_name(name), m_params(params), m_parenLevel(0) {}
     ~VPreDefRef() {}
 };
 
@@ -118,14 +130,19 @@ struct VPreprocImp : public VPreprocOpaque {
 	m_rawAtBol = true;
 	m_defDepth = 0;
     }
+    ~VPreprocImp() {
+	if (m_lexp) { delete m_lexp; m_lexp = NULL; }
+    }
     const char* tokenName(int tok);
     int getRawToken();
     int getToken();
+    void debugToken(int tok, const char* cmtp);
     void parseTop();
     void parseUndef();
     string getparseline(bool stop_at_eol);
     bool isEof() const { return (m_lexp==NULL); }
-    void open(string filename, VFileLine* filelinep);
+    bool readWholefile(const string& filename, string& out);
+    void openFile(string filename, VFileLine* filelinep);
     void insertUnreadback(const string& text) { m_lineCmt += text; }
     void insertUnreadbackAtBol(const string& text);
 private:
@@ -135,8 +152,8 @@ private:
     void eof();
     string defineSubst(VPreDefRef* refp);
     void addLineComment(int enter_exit_level);
-    string trimWhitespace(const string& strg);
-    void unputString(const string& strg);
+    string trimWhitespace(const string& strg, bool trailing);
+    void unputString(const string& strg, bool first=false);
 
     void parsingOn() { m_off--; assert(m_off>=0); if (!m_off) addLineComment(0); }
     void parsingOff() { m_off++; }
@@ -151,13 +168,17 @@ VPreproc::VPreproc(VFileLine* filelinep) {
     idatap->m_preprocp = this;
 }
 
+VPreproc::~VPreproc() {
+    if (m_opaquep) { delete m_opaquep; m_opaquep = NULL; }
+}
+
 //*************************************************************************
 // VPreproc Methods.  Just call the implementation functions.
 
 void VPreproc::comment(string cmt) { }
-void VPreproc::open(string filename, VFileLine* filelinep) {
+void VPreproc::openFile(string filename, VFileLine* filelinep) {
     VPreprocImp* idatap = static_cast<VPreprocImp*>(m_opaquep);
-    idatap->open (filename,filelinep);
+    idatap->openFile (filename,filelinep);
 }
 string VPreproc::getline() {
     VPreprocImp* idatap = static_cast<VPreprocImp*>(m_opaquep);
@@ -191,7 +212,7 @@ void VPreproc::insertUnreadback(string text) {
 // This probably will want to be overridden for given child users of this class.
 
 void VPreproc::include(string filename) {
-    open(filename, filelinep());
+    openFile(filename, filelinep());
 }
 void VPreproc::undef(string define) {
     cout<<"UNDEF "<<define<<endl;
@@ -247,28 +268,40 @@ const char* VPreprocImp::tokenName(int tok) {
     }
 }
 
-void VPreprocImp::unputString(const string& strg) {
+void VPreprocImp::unputString(const string& strg, bool first) {
     // We used to just m_lexp->unputString(strg.c_str());
     // However this can lead to "flex scanner push-back overflow"
     // so instead we scan from a temporary buffer, then on EOF return.
     // This is also faster than the old scheme, amazingly.
-    if (m_lexp->m_bufferStack.empty() || m_lexp->m_bufferStack.top()!=m_lexp->currentBuffer()) {
-	fatalSrc("bufferStack missing current buffer; will return incorrectly");
-	// Hard to debug lost text as won't know till much later
+    if (!first) {  // Else the initial creation
+	if (m_lexp->m_bufferStack.empty() || m_lexp->m_bufferStack.top()!=m_lexp->currentBuffer()) {
+	    fatalSrc("bufferStack missing current buffer; will return incorrectly");
+	    // Hard to debug lost text as won't know till much later
+	}
     }
     m_lexp->scanBytes(strg);
 }
 
-string VPreprocImp::trimWhitespace(const string& strg) {
+string VPreprocImp::trimWhitespace(const string& strg, bool trailing) {
+    // Remove leading whitespace
     string out = strg;
-    while (out.length()>0 && isspace(out[0])) {
-	out.erase(0,1);
+    string::size_type leadspace = 0;
+    while (out.length() > leadspace
+	   && isspace(out[leadspace])) leadspace++;
+    if (leadspace) out.erase(0,leadspace);
+    // Remove trailing whitespace
+    if (trailing) {
+	string::size_type trailspace = 0;
+	while (out.length() > trailspace
+	       && isspace(out[out.length()-1-trailspace])) trailspace++;
+	if (trailspace) out.erase(out.length()-trailspace,trailspace);
     }
     return out;
 }
 
 string VPreprocImp::defineSubst(VPreDefRef* refp) {
-    // Substitute out defines in a argumented define reference.
+    // Substitute out defines in a define reference.
+    // (We also need to call here on non-param defines to handle `")
     // We could push the define text back into the lexer, but that's slow
     // and would make recursive definitions and parameter handling nasty.
     //
@@ -278,7 +311,7 @@ string VPreprocImp::defineSubst(VPreDefRef* refp) {
     if (debug()) {
 	cout<<"defineSubstIn  `"<<refp->name()<<" "<<refp->params()<<endl;
 	for (unsigned i=0; i<refp->args().size(); i++) {
-	    cout<<"defineArg["<<i<<"] = "<<refp->args()[i]<<endl;
+	    cout<<"defineArg["<<i<<"] = '"<<refp->args()[i]<<"'"<<endl;
 	}
     }
     // Grab value
@@ -289,27 +322,65 @@ string VPreprocImp::defineSubst(VPreDefRef* refp) {
     {   // Parse argument list into map
 	unsigned numArgs=0;
 	string argName;
-	for (const char* cp=refp->params().c_str(); *cp; cp++) {
-	    if (*cp=='(') {
-	    } else if (argName=="" && isspace(*cp)) {
-	    } else if (isspace(*cp) || *cp==')' || *cp==',') {
-		if (argName!="") {
-		    if (refp->args().size() > numArgs) {
-			// A call `def( a ) must be equivelent to `def(a ), so trimWhitespace
-			// Note other sims don't trim trailing whitespace, so we don't either.
-			argValueByName[argName] = trimWhitespace(refp->args()[numArgs]);
+	int paren = 1;  // (), {} and [] can use same counter, as must be matched pair per spec
+	string token;
+	bool quote = false;
+	bool haveDefault = false;
+	// Note there's a leading ( and trailing ), so parens==1 is the base parsing level
+	const char* cp=refp->params().c_str();
+	if (*cp == '(') cp++;
+	for (; *cp; cp++) {
+	    //if (debug()) cout <<"   Parse  Paren="<<paren<<"  Arg="<<numArgs<<"  token='"<<token<<"'  Parse="<<cp<<endl;
+	    if (!quote && paren==1) {
+		if (*cp==')' || *cp==',') {
+		    string value;
+		    if (haveDefault) { value=token; } else { argName=token; }
+		    argName = trimWhitespace(argName,true);
+		    if (debug()) cout<<"    Got Arg="<<numArgs<<"  argName='"<<argName<<"'  default='"<<value<<"'"<<endl;
+		    // Parse it
+		    if (argName!="") {
+			if (refp->args().size() > numArgs) {
+			    // A call `def( a ) must be equivelent to `def(a ), so trimWhitespace
+			    // Note other sims don't trim trailing whitespace, so we don't either.
+			    string arg = trimWhitespace(refp->args()[numArgs], false);
+			    if (arg != "") value = arg;
+			} else if (!haveDefault) {
+			    error("Define missing argument '"+argName+"' for: "+refp->name()+"\n");
+			    return " `"+refp->name()+" ";
+			}
+			numArgs++;
 		    }
-		    numArgs++;
-		    //cout << "  arg "<<argName<<endl;
+		    argValueByName[argName] = value;
+		    // Prepare for next
+		    argName = "";
+		    token = "";
+		    haveDefault = false;
+		    continue;
+		}
+		else if (*cp=='=') {
+		    haveDefault = true;
+		    argName = token;
+		    token = "";
+		    continue;
 		}
-		argName = "";
-	    } else if ( isalpha(*cp) || *cp=='_'
-			|| (argName!="" && (isdigit(*cp) || *cp=='$'))) {
-		argName += *cp;
 	    }
+	    if (cp[0]=='\\' && cp[1]) {
+		token += cp[0]; // \{any} Put out literal next character
+		token += cp[1];
+		cp++;
+		continue;
+	    }
+	    if (!quote) {
+		if (*cp=='(' || *cp=='{' || *cp=='[') paren++;
+		else if (*cp==')' || *cp=='}' || *cp==']') paren--;
+	    }
+	    if (*cp=='"') quote=!quote;
+	    if (*cp) token += *cp;
 	}
-	if (refp->args().size() != numArgs) {
-	    error("Define passed wrong number of arguments: "+refp->name()+"\n");
+	if (refp->args().size() > numArgs
+	    // `define X() is ok to call with nothing
+	    && !(refp->args().size()==1 && numArgs==0 && trimWhitespace(refp->args()[0],false)=="")) {
+	    error("Define passed too many arguments: "+refp->name()+"\n");
 	    return " `"+refp->name()+" ";
 	}
     }
@@ -378,18 +449,51 @@ string VPreprocImp::defineSubst(VPreDefRef* refp) {
 //**********************************************************************
 // Parser routines
 
-void VPreprocImp::open(string filename, VFileLine* filelinep) {
+bool VPreprocImp::readWholefile(const string& filename, string& out) {
+    int fd = open (filename.c_str(), O_RDONLY);
+    if (!fd) return false;
+
+#define INFILTER_IPC_BUFSIZ 64*1024
+    char buf[INFILTER_IPC_BUFSIZ];
+    bool eof = false;
+    while (!eof) {
+	int todo = INFILTER_IPC_BUFSIZ;
+	int got = read (fd, buf, todo);
+	if (got>0) out.append(buf, got);
+	else if (errno == EINTR || errno == EAGAIN
+#ifdef EWOULDBLOCK
+		 || errno == EWOULDBLOCK
+#endif
+	    ) {
+	} else { eof = true; break; }
+    }
+
+    close(fd);
+    return true;
+}
+
+void VPreprocImp::openFile(string filename, VFileLine* filelinep) {
     // Open a new file, possibly overriding the current one which is active.
     if (filelinep) {
 	m_filelinep = filelinep;
     }
 
-    FILE* fp = fopen (filename.c_str(), "r");
-    if (!fp) {
+    string wholefile;
+    bool ok = readWholefile(filename, wholefile/*ref*/);
+    if (!ok) {
 	error("File not found: "+filename+"\n");
 	return;
     }
 
+    // Filter all DOS CR's en-mass.  This avoids bugs with lexing CRs in the wrong places.
+    // This will also strip them from strings, but strings aren't supposed to be multi-line without a "\"
+    string wholefilecr;
+    size_t wholesize = wholefile.length();
+    for (size_t i=0; i<wholesize; i++) {  // Not a c_str(), as we keep '\0's for now.
+	if (wholefile[i] != '\r' && wholefile[i] != '\0') wholefilecr += wholefile[i];
+    }
+    wholefile.resize(0); // free memory
+
     if (m_lexp) {
 	// We allow the same include file twice, because occasionally it pops
 	// up, with guards preventing a real recursion.
@@ -402,13 +506,15 @@ void VPreprocImp::open(string filename, VFileLine* filelinep) {
 	addLineComment(0);
     }
 
-    m_lexp = new VPreprocLex (fp);
+    m_lexp = new VPreprocLex ();
     m_lexp->m_keepComments = m_preprocp->keepComments();
     m_lexp->m_keepWhitespace = m_preprocp->keepWhitespace();
     m_lexp->m_pedantic = m_preprocp->pedantic();
     m_lexp->m_curFilelinep = m_preprocp->filelinep()->create(filename, 1);
     m_filelinep = m_lexp->m_curFilelinep;  // Remember token start location
     addLineComment(1); // Enter
+
+    unputString(wholefilecr,true);
 }
 
 void VPreprocImp::insertUnreadbackAtBol(const string& text) {
@@ -465,7 +571,8 @@ int VPreprocImp::getRawToken() {
 	    m_lineAdd--;
 	    m_rawAtBol = true;
 	    yytext=(char*)"\n"; yyleng=1;
-	    return (VP_TEXT);
+	    if (debug()) debugToken(VP_WHITE, "LNA");
+	    return (VP_WHITE);
 	}
 	if (m_lineCmt!="") {
 	    // We have some `line directive or other processed data to return to the user.
@@ -482,6 +589,7 @@ int VPreprocImp::getRawToken() {
 		VPreprocLex::s_currentLexp->appendDefValue(yytext,yyleng);
 		goto next_tok;
 	    } else {
+		if (debug()) debugToken(VP_TEXT, "LCM");
 		return (VP_TEXT);
 	    }
 	}
@@ -492,15 +600,7 @@ int VPreprocImp::getRawToken() {
 	VPreprocLex::s_currentLexp = m_lexp;   // Tell parser where to get/put data
 	int tok = yylex();
 
-	if (debug()) {
-	    string buf = string (yytext, yyleng);
-	    string::size_type pos;
-	    while ((pos=buf.find("\n")) != string::npos) { buf.replace(pos, 1, "\\n"); }
-	    while ((pos=buf.find("\r")) != string::npos) { buf.replace(pos, 1, "\\r"); }
-	    fprintf (stderr, "%d: RAW %s s%d dr%d:  <%d>%-10s: %s\n",
-		     m_filelinep->lineno(), m_off?"of":"on", m_state, (int)m_defRefs.size(),
-		     m_lexp->currentStartState(), tokenName(tok), buf.c_str());
-	}
+	if (debug()) debugToken(tok, "RAW");
 
 	// On EOF, try to pop to upper level includes, as needed.
 	if (tok==VP_EOF) {
@@ -513,6 +613,18 @@ int VPreprocImp::getRawToken() {
     }
 }
 
+void VPreprocImp::debugToken(int tok, const char* cmtp) {
+    if (debug()) {
+	string buf = string (yytext, yyleng);
+	string::size_type pos;
+	while ((pos=buf.find("\n")) != string::npos) { buf.replace(pos, 1, "\\n"); }
+	while ((pos=buf.find("\r")) != string::npos) { buf.replace(pos, 1, "\\r"); }
+	fprintf (stderr, "%d: %s %s s%d dr%d:  <%d>%-10s: %s\n",
+		 m_filelinep->lineno(), cmtp, m_off?"of":"on", m_state, (int)m_defRefs.size(),
+		 m_lexp->currentStartState(), tokenName(tok), buf.c_str());
+    }
+}
+
 // Sorry, we're not using bison/yacc. It doesn't handle returning white space
 // in the middle of parsing other tokens.
 
@@ -590,6 +702,11 @@ int VPreprocImp::getToken() {
 		else fatalSrc("Bad case\n");
 		goto next_tok;
 	    }
+	    else if (tok==VP_TEXT) {
+		// IE, something like comment between define and symbol
+		if (!m_off) return tok;
+		else goto next_tok;
+	    }
 	    else {
 		error((string)"Expecting define name. Found: "+tokenName(tok)+"\n");
 		goto next_tok;
@@ -617,45 +734,35 @@ int VPreprocImp::getToken() {
 	    if (tok == VP_DEFVALUE) {
 		if (debug()) cout<<"DefValue='"<<m_lexp->m_defValue<<"'  formals='"<<m_formals<<"'\n";
 		// Add any formals
-		string formAndValue = m_formals + m_lexp->m_defValue;
+		string formals = m_formals;
+		string value = m_lexp->m_defValue;
 		// Remove returns
-		for (unsigned i=0; i<formAndValue.length(); i++) {
-		    if (formAndValue[i] == '\n') {
-			formAndValue[i] = ' ';
+		// Not removing returns in values has two problems,
+		// 1. we need to correct line numbers with `line after each substitution
+		// 2. Substituting in " .... " with embedded returns requires \ escape.
+		//    This is very difficult in the presence of `".
+		for (size_t i=0; i<formals.length(); i++) {
+		    if (formals[i] == '\n') {
+			formals[i] = ' ';
+			newlines += "\n";
+		    }
+		}
+		for (size_t i=0; i<value.length(); i++) {
+		    if (value[i] == '\n') {
+			value[i] = ' ';
 			newlines += "\n";
 		    }
 		}
 		if (!m_off) {
-		    string params;
-		    if (formAndValue=="" || isspace(formAndValue[0])) {
-			// Define without parameters
-		    } else if (formAndValue[0] == '(') {
-			string::size_type paren = formAndValue.find(")");
-			if (paren == string::npos) {
-			    error((string)"Missing ) to end define arguments.\n");
-			} else {
-			    params = formAndValue.substr(0, paren+1);
-			    formAndValue.replace(0, paren+1, "");
-			}
-		    } else {
-			error((string)"Missing space or paren to start define value.\n");
-		    }
-		    // Remove leading whitespace
-		    unsigned leadspace = 0;
-		    while (formAndValue.length() > leadspace
-			   && isspace(formAndValue[leadspace])) leadspace++;
-		    if (leadspace) formAndValue.erase(0,leadspace);
-		    // Remove trailing whitespace
-		    unsigned trailspace = 0;
-		    while (formAndValue.length() > trailspace
-			   && isspace(formAndValue[formAndValue.length()-1-trailspace])) trailspace++;
-		    if (trailspace) formAndValue.erase(formAndValue.length()-trailspace,trailspace);
+		    // Remove leading and trailing whitespace
+		    value = trimWhitespace(value, true);
 		    // Define it
-		    if (debug()) cout<<"Define "<<m_lastSym<<" = '"<<formAndValue<<"'"<<endl;
-		    m_preprocp->define(m_lastSym, formAndValue, params);
+		    if (debug()) cout<<"Define "<<m_lastSym<<" "<<formals<<" = '"<<value<<"'"<<endl;
+		    m_preprocp->define(m_lastSym, value, formals);
 		}
 	    } else {
-		fatalSrc("Bad define text\n");
+		string msg = string("Bad define text, unexpected ")+tokenName(tok)+"\n";
+		fatalSrc(msg);
 	    }
 	    m_state = ps_TOP;
 	    // DEFVALUE is terminated by a return, but lex can't return both tokens.
@@ -839,7 +946,9 @@ int VPreprocImp::getToken() {
 		}
 		else {  // Found, with parameters
 		    if (debug()) cout<<"Defref `"<<name<<" => parametrized"<<endl;
-		    m_defRefs.push(VPreDefRef(name, params, m_lexp->m_parenLevel));
+		    // The CURRENT macro needs the paren saved, it's not a property of the child macro
+		    if (!m_defRefs.empty()) m_defRefs.top().parenLevel(m_lexp->m_parenLevel);
+		    m_defRefs.push(VPreDefRef(name, params));
 		    m_state = ps_DEFPAREN;  m_stateFor = tok;
 		    m_lexp->pushStateDefArg(0);
 		    goto next_tok;
